@@ -2,7 +2,7 @@ __author__ = 'Altertech, https://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2019 Altertech'
 __license__ = 'MIT'
 
-__version__ = '0.4.24'
+__version__ = '0.4.29'
 
 from sqlalchemy.exc import IntegrityError
 from cachetools import TTLCache
@@ -264,8 +264,6 @@ def exec_query(q, _grafana=False):
     """
     Execute statement
 
-    EXPERIMENTAL, DON'T USE IN PRODUCTION
-
     Currenly only function call statements are supported:
 
         SELECT <function>([args, kwargs])
@@ -819,19 +817,26 @@ def asset_delete_rate(asset_from, asset_to=None, date=None):
 
 
 def _parse_asset_pair(asset_from, asset_to):
+    if asset_from is None:
+        raise ValueError('source asset not specified')
     if asset_from.find('/') != -1 and asset_to is None:
         asset_from, asset_to = asset_from.split('/')
+    elif asset_to is None:
+        raise ValueError('target asset not specified')
     asset_from = asset_from.upper()
     asset_to = asset_to.upper()
     return asset_from, asset_to
 
 
 @core_method
-def asset_rate(asset_from,
+def asset_rate(asset_from=None,
                asset_to=None,
                date=None,
                _grafana=False,
+               asset=None,
                return_pair=False):
+    if asset:
+        asset_from = asset
     asset_from, asset_to = _parse_asset_pair(asset_from, asset_to)
     result = _asset_rate_lookup(asset_from,
                                 asset_to,
@@ -908,8 +913,11 @@ def _asset_rate_lookup(asset_from, asset_to=None, date=None, _grafana=False):
             graph.setdefault(r['asset_from'], []).append(r['asset_to'])
         for k, v in rates.copy().items():
             if (k[1], k[0]) not in rates:
-                rates[(k[1], k[0])] = 1 / v
-                graph.setdefault(k[1], []).append(k[0])
+                try:
+                    rates[(k[1], k[0])] = 1 / v
+                    graph.setdefault(k[1], []).append(k[0])
+                except ZeroDivisionError:
+                    pass
         try:
             path = min(_find_path(graph, asset_from, asset_to), key=len)
         except (KeyError, ValueError):
@@ -2249,7 +2257,7 @@ def account_balance(account=None,
 
     Args:
         account: account code
-        tp: account type/types
+        tp: account type/types, value, list or values, separated with |
         base: base asset (if not specified, config.base_asset is used)
         date: get balance for specified date/time
     """
@@ -2306,43 +2314,34 @@ def account_balance(account=None,
 
 @core_method
 def asset_rate_range(start,
-                     asset_from,
+                     asset_from=None,
                      asset_to=None,
                      end=None,
                      step=1,
+                     asset=None,
                      return_timestamp=False,
                      _grafana=False):
     """
     Get list of asset rates for the specified time range
 
+    step argument usage:
+
+        4, 4d - 4 days
+        2h - 2 hours
+        5a - split time range into 5 parts
+
     Returns:
         tuple with time series list and corresponding asset rate
     """
-    if _grafana:
-        if isinstance(start, int):
-            start = start / 1000
-        if isinstance(end, int):
-            end = end / 1000
-    times = []
-    data = []
-    dt = parse_date(start, return_timestamp=False)
-    end_date = parse_date(
-        end, return_timestamp=False) if end else datetime.datetime.now()
-    end_date += datetime.timedelta(days=1)
-    delta = datetime.timedelta(days=step)
-    last_record = False
-    while dt < end_date or not last_record:
-        if dt == end_date:
-            break
-        elif dt > end_date:
-            last_record = True
-            if _grafana:
-                dt = datetime.datetime.now()
-        times.append(dt.timestamp() if return_timestamp else dt)
-        b = _asset_rate_lookup(asset_from, asset_to, date=dt)
-        data.append(b)
-        dt += delta
-    return times, data
+    if asset:
+        asset_from = asset
+    return _run_steps_func(start=start,
+                           end=end,
+                           step=step,
+                           return_timestamp=return_timestamp,
+                           fn=_asset_rate_lookup,
+                           args=(asset_from, asset_to),
+                           _grafana=_grafana)
 
 
 @core_method
@@ -2357,12 +2356,18 @@ def account_balance_range(start,
     """
     Get list of account balances for the specified time range
 
+    step argument usage:
+
+        4, 4d - 4 days
+        2h - 2 hours
+        5a - split time range into 5 parts
+
     Args:
         account: account code
         tp: account type/types
         start: start date/time, required
         end: end date/time, if not specified, current time is used
-        step: list step in days
+        step: time step
         return_timestamp: return dates as timestamps if True, otherwise as
             datetime objects. Default is False
 
@@ -2375,6 +2380,26 @@ def account_balance_range(start,
         tp = [k for k in ACCOUNT_TYPE_IDS if ACCOUNT_TYPE_IDS[k] <= 1000]
     elif tp and '|' in tp:
         tp = [x.strip() for x in tp.split('|')]
+    acc_info = {'account': account} if account else {'tp': tp}
+    return _run_steps_func(start=start,
+                           end=end,
+                           step=step,
+                           return_timestamp=return_timestamp,
+                           fn=account_balance,
+                           kwargs={
+                               **acc_info, 'base': base
+                           },
+                           _grafana=_grafana)
+
+
+def _run_steps_func(start,
+                    end,
+                    step,
+                    return_timestamp,
+                    fn,
+                    args=(),
+                    kwargs={},
+                    _grafana=False):
     if _grafana:
         if isinstance(start, int):
             start = start / 1000
@@ -2382,23 +2407,29 @@ def account_balance_range(start,
             end = end / 1000
     times = []
     data = []
-    acc_info = {'account': account} if account else {'tp': tp}
     dt = parse_date(start, return_timestamp=False)
     end_date = parse_date(
         end, return_timestamp=False) if end else datetime.datetime.now()
-    delta = datetime.timedelta(days=step)
-    end_date += datetime.timedelta(days=1)
-    last_record = False
-    while dt < end_date or not last_record:
-        if dt == end_date:
-            break
-        elif dt > end_date:
-            last_record = True
-            if _grafana:
-                dt = datetime.datetime.now()
+    if isinstance(step, str) and step.endswith('a'):
+        step = int(step[:-1])
+        delta = (end_date - dt) / (step - 1) if step > 1 else None
+        autosteps = True
+    else:
+        autosteps = False
+        if isinstance(step, str):
+            if step.endswith('h'):
+                delta = datetime.timedelta(hours=int(step[:-1]))
+            elif step.endswith('d'):
+                delta = datetime.timedelta(days=int(step[:-1]))
+            else:
+                delta = datetime.timedelta(days=int(step))
+        else:
+            delta = datetime.timedelta(days=step)
+    while dt <= end_date or (autosteps and len(data) < step):
         times.append(dt.timestamp() if return_timestamp else dt)
-        b = account_balance(**acc_info, base=base, date=dt)
-        data.append(b)
+        data.append(fn(*args, date=dt, **kwargs))
+        if delta is None:
+            break
         dt += delta
     return times, data
 
